"""
シミュレーション時間とTIME_STEPはstep応答、ramp応答で揃えること
step入力は5秒後から開始 ramp入力は0~20で20秒間
end_time = 20
TIME_STEP = 1e-4 * 5
運動閾値：初期角度+0.5°
最大角度：最終角度-0.5°
"""

import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import sys
import glob

condition = 1  # 0：BIC、1：TRI
if condition == 0:
    muslce = "BIC"
elif condition == 1:
    muslce = "TRI"
folder_path = r'\\Fesback\fes\Ryusuke_Yokoya\M2\osim\Results\0731\for_PID\default\{}'.format(muslce)

# ワイルドカードを使ってファイルを見つける
step_files = glob.glob(os.path.join(folder_path, '1_Step_*.csv'))
ramp_files = glob.glob(os.path.join(folder_path, '1_Ramp_*.csv'))

# 最初の一致ファイルを読み込む（複数ある場合は適宜変更）
if step_files:
    step_file_path = step_files[0]
else:
    print('Stepファイルが見つかりません。')
    sys.exit()

if ramp_files:
    ramp_file_path = ramp_files[0]
else:
    print('Rampファイルが見つかりません。')
    sys.exit()

Step = pd.read_csv(step_file_path)
Ramp = pd.read_csv(ramp_file_path)

# 以下、元のコードの続き
Time = 'Time'
if condition == 0:
    Exc = "Excitation_BIC"
elif condition == 1:
    Exc = "Excitation_TRI"
Angle = "ObservedAngle"
time_step = Step["TIME_STEP"][0]
if time_step != Ramp["TIME_STEP"][0]:
    print('Step応答とRamp応答のtime_stepが異なります。プログラムを中断します。')
    sys.exit()

ta = int(5 / time_step)  # 変曲点探索の範囲
t = np.arange(0, Step["end_time"][0] + time_step, time_step)

T_Step_Start = Step[Step[Exc] > 0.01].index[0]
T_Step_Finish = Step[Step[Exc] > 0.01].index[-1]
T_Ramp_Start = Ramp[Ramp[Exc] > 0.01].index[0]
T_Ramp_Finish = Ramp[Ramp[Exc] > 0.01].index[-1]

# ステップ応答からL,T決定
delta = [Step[Angle][i] - Step[Angle][i - 1] for i in range(T_Step_Start, T_Step_Finish)]
if condition == 0:
    max_delta = max(delta[:ta])
elif condition == 1:
    max_delta = min(delta[:ta])
ta = delta.index(max_delta)  # 指定した範囲における傾きの最大値（変曲点）
slope = max_delta / time_step  # 変曲点における傾き
t0 = Step[Time][T_Step_Start + ta] - ((Step[Angle][T_Step_Start + ta] - Step[Angle][T_Step_Start]) / slope)  # 変曲点における接線とx軸の交点
t1 = Step[Time][T_Step_Start + ta] - ((Step[Angle][T_Step_Start + ta] - Step[Angle][T_Step_Finish]) / slope)
L = t0 - Step[Time][T_Step_Start]  # 無駄時間
T = t1 - t0  # 時定数

# 閾値インデックスと最終インデックスの計算
if condition == 0:
    threshold_start = Ramp[Angle][0] + 0.5
    threshold_end = Ramp[Angle].iloc[-1] - 0.5
    threshold_index = Ramp[Ramp[Angle] > threshold_start].index[0]
    final_index = Ramp[Ramp[Angle] > threshold_end].index[0]
elif condition == 1:
    threshold_start = Ramp[Angle][0] - 0.5
    threshold_end = Ramp[Angle].iloc[-1] + 0.5
    threshold_index = Ramp[Ramp[Angle] < threshold_start].index[0]
    final_index = Ramp[Ramp[Angle] < threshold_end].index[0]

# ランプ応答の線形近似
p = np.polyfit(Ramp[Exc][threshold_index:final_index], Ramp[Angle][threshold_index:final_index], 1)
p1 = np.polyval(p, Ramp[Exc][threshold_index:final_index])
p[0] = np.radians(p[0])

Kp = 0.6 * T / L / p[0]
Ki = 0.6 / L / p[0]
Kd = 0.3 * T / p[0]

print(f"excitation_threshold: {Ramp[Exc][threshold_index]}")
print(f"excitation_max: {Ramp[Exc][final_index]}")
print(f"L: {L} ")
print(f"T: {T} ")
print(f"m: {p[0]} ")
print(f"Kp: {Kp} ")
print(f"Ki: {Ki} ")
print(f"Kd: {Kd} ")

# 以下グラフプロット

# t0に一番近いtime_stepの倍数
t00 = round(t0 / time_step) * time_step
# (t0+T)に一番近いtime_stepの倍数
t00_T = round((t0 + T) / time_step) * time_step
t2 = np.arange(t00, t00_T, time_step)
a = []
for j in range(len(t2)):
    x = slope * (t2[j] - (T_Step_Start + ta) * time_step) + Step[Angle][T_Step_Start + ta]
    a.append(x)

plt.figure(1)

# 左軸（肘関節角度）のプロット
ax1 = plt.gca()  # 現在の軸を取得
ax1.plot(t, Step[Angle], 'b-', label="Step Response")  # 肘関節角度を青色でプロット
ax1.plot(t2, a, 'g--', label="Tangent")  # 接線を緑の点線でプロット
ax1.set_xlabel('Time [s]')
ax1.set_ylabel('Elbow Joint Angle [deg]', color='b')  # y軸のラベルと色
ax1.tick_params('y', colors='b')  # y軸の色
ax1.legend(loc='center right')  # 凡例の位置

# 右軸（刺激強度）のプロット
ax2 = ax1.twinx()  # 同じx軸を共有する別のy軸を作成
ax2.plot(t, Step[Exc], 'r-', label="Excitation")  # 刺激強度を赤色でプロット
ax2.set_ylabel('Excitation', color='r')  # y軸のラベルと色
ax2.tick_params('y', colors='r')  # y軸の色
ax2.legend(loc='center')  # 凡例の位置

plt.figure(2)
plt.plot(Ramp[Exc][threshold_index:final_index], Ramp[Angle][threshold_index:final_index])
plt.plot(Ramp[Exc][threshold_index:final_index], p1, 'r--')
plt.xlabel('Excitation')
plt.ylabel('Elbow Joint Angle [deg]')
plt.show()
