import tkinter as tk
from tkinter import ttk, filedialog, messagebox, scrolledtext
import os
import numpy as np
import datetime
from control_main import control_main
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg

app = tk.Tk()
app.title("Simulation Control Panel")

quiet_time_var = tk.StringVar()
end_time_var = tk.StringVar()
excitation_BIC_max_var = tk.StringVar()
excitation_TRI_max_var = tk.StringVar()

def run_simulation():
    try:
        total_iteration = int(total_iteration_entry.get())
        control_condition = control_condition_dict[control_condition_combobox.get()]
        target_muscle = target_muscle_dict[target_muscle_combobox.get()]
        excitation_BIC_max = float(excitation_BIC_max_var.get())
        excitation_TRI_max = float(excitation_TRI_max_var.get())
        quiet_time = int(quiet_time_var.get())
        end_time = int(end_time_var.get())
        new_folder_name = new_folder_name_entry.get()
        TIME_STEP = eval(time_step_entry.get())
        user_notes = notes_text.get("1.0", "end-1c")

        script_directory_path = os.path.dirname(__file__)
        model_type = model_combobox.get()
        model_file_name = "arm26.osim"
        model_folder_name = os.path.join(script_directory_path, "../../model", model_type)
        IniSta_file_name = "InitialState.sto"
        IniSta_folder_name = model_folder_name
        result_folder_name = os.path.join("Results", new_folder_name)

        iteration = 1
        iteration_flag = True
        training_history = np.zeros(22)
        bug_flag, training_history = control_main(
            control_condition, target_muscle, 5, quiet_time, end_time,
            excitation_BIC_max, excitation_TRI_max, 0.06, 0.05, 1/10,
            model_file_name, model_folder_name, IniSta_file_name,
            IniSta_folder_name, result_folder_name, iteration_flag,
            iteration, total_iteration, training_history, TIME_STEP
        )
        if not bug_flag:
            messagebox.showinfo("Success", "Simulation completed successfully.\nNotes saved.")
        else:
            messagebox.showerror("Error", "An error occurred during the simulation.")
    except Exception as e:
        messagebox.showerror("Error", f"An error occurred: {e}")

def draw_waveform():
    selected_condition = control_condition_combobox.get()
    quiet_time = int(quiet_time_var.get()) if quiet_time_var.get() else 0
    end_time = int(end_time_var.get()) if end_time_var.get() else 1
    excitation_BIC_max = float(excitation_BIC_max_var.get()) if excitation_BIC_max_var.get() else 0
    excitation_TRI_max = float(excitation_TRI_max_var.get()) if excitation_TRI_max_var.get() else 0
    
    t = np.linspace(0, end_time, 500)  # 時間軸

    # デフォルト値を設定
    excitation_BIC = np.full_like(t, 0.01)  # 全てのtで0.01とする
    excitation_TRI = np.full_like(t, 0.01)  # 全てのtで0.01とする

    if selected_condition == "Step":
        excitation_BIC = np.where(t <= quiet_time, 0.01, excitation_BIC_max)
        excitation_TRI = np.where(t <= quiet_time, 0.01, excitation_TRI_max)     
    elif selected_condition == "Ramp":
        excitation_BIC = np.where(t <= quiet_time, 0.01, np.interp(t, [quiet_time, end_time], [0.01, excitation_BIC_max]))
        excitation_TRI = np.where(t <= quiet_time, 0.01, np.interp(t, [quiet_time, end_time], [0.01, excitation_TRI_max]))
    elif selected_condition == "三角波":
        excitation_BIC = np.where(
            t <= quiet_time,
            0.01,
            np.where(
                t <= end_time / 2,
                np.interp(t, [quiet_time, end_time / 2], [0.01, excitation_BIC_max]),
                np.where(
                    t <= (end_time - quiet_time),
                    np.interp(t, [end_time / 2, (end_time - quiet_time)], [excitation_BIC_max, 0.01]),
                    0.01
                )
            )
        )

    # グラフの描画
    ax.clear()
    ax.plot(t, excitation_BIC, label='BIC')
    ax.plot(t, excitation_TRI, label='TRI')

    ax.grid(True)
    if selected_condition in ["Step", "Ramp"]:
        ax.set_xticks([0, quiet_time, end_time])
    elif selected_condition == "三角波":
        ax.set_xticks([0, quiet_time, end_time / 2, end_time - quiet_time, end_time])
    ax.set_yticks([0, 0.01, excitation_BIC_max, excitation_TRI_max])
    
    # 軸のラベルとタイトルの設定
    ax.set_xlabel('Time (s)')
    ax.set_ylabel('Excitation')
    ax.set_title('Waveform Preview')

    # 凡例の追加
    ax.legend()

    # キャンバスの更新
    canvas.draw()

    # キャンバスを再配置（以前のキャンバスウィジェットがすでにある場合は削除する）
    canvas_widget = canvas.get_tk_widget()
    canvas_widget.grid(column=2, row=0, rowspan=10, sticky="nsew")

def update_time(event):
    selected_condition = control_condition_combobox.get()
    if selected_condition in ["Step", "Ramp", "三角波"]:
        quiet_time_var.set("1")
    elif selected_condition in ["PID", "FEL"]:
        quiet_time_var.set("0")
    else:
        quiet_time_var.set("")
    if selected_condition == "Step":
        end_time_var.set("6")
    elif selected_condition == "Ramp":
        end_time_var.set("21")
    elif selected_condition == "三角波":
        end_time_var.set("22")
    elif selected_condition in ["PID", "FEL"]:
        end_time_var.set("28")

    draw_waveform()

def on_value_change(*args):
    draw_waveform()

# グラフを表示するための Figure と Canvas を作成
fig = plt.Figure(figsize=(6, 5), dpi=100)
ax = fig.add_subplot(111)
canvas = FigureCanvasTkAgg(fig, master=app)  # app は Tkinter の root ウィンドウ
canvas.get_tk_widget().grid(column=2, row=0, rowspan=10, sticky="nsew")

# Adjust the default font size and the uniform width for widgets
default_font = ('Helvetica', 15)
uniform_width = 20  # Setting a uniform width for all input and dropdown fields
app.option_add("*Font", default_font)

# Set a uniform width for entry and combobox widgets
uniform_width = 20

# Dictionaries for combobox values
control_condition_dict = {
    "Step": 0, "Ramp": 1, "PID": 2, "Ramp2Step": 3,
    "Sin_Wave": 5, "FEL": 6, "三角波": 7
}
target_muscle_dict = {"BIC": 0, "TRI": 1, "BICTRI": 2}
model_list = ["default", "type1", "fat", "thin", "powerful", "weak", "athletic", "sedentary", "elderly"]

quiet_time_entry = ttk.Entry(app, textvariable=quiet_time_var, width=uniform_width)
end_time_entry = ttk.Entry(app, textvariable=end_time_var, width=uniform_width)
excitation_BIC_max_entry = ttk.Entry(app, textvariable=excitation_BIC_max_var, width=uniform_width)
excitation_TRI_max_entry = ttk.Entry(app, textvariable=excitation_TRI_max_var, width=uniform_width)

# Create and place widgets for each parameter with labels aligned left and uniform width
total_iteration_label = ttk.Label(app, text="Total Iteration:", anchor="w")
total_iteration_label.grid(column=0, row=0, sticky="w")
total_iteration_entry = ttk.Entry(app, width=uniform_width)
total_iteration_entry.grid(column=1, row=0)
total_iteration_entry.insert(0, "1")

control_condition_label = ttk.Label(app, text="Control Condition:", anchor="w")
control_condition_label.grid(column=0, row=1, sticky="w")
control_condition_combobox = ttk.Combobox(app, values=list(control_condition_dict.keys()), state="readonly", width=uniform_width)
control_condition_combobox.grid(column=1, row=1)
control_condition_combobox.bind("<<ComboboxSelected>>", update_time)

target_muscle_label = ttk.Label(app, text="Target Muscle:", anchor="w")
target_muscle_label.grid(column=0, row=2, sticky="w")
target_muscle_combobox = ttk.Combobox(app, values=list(target_muscle_dict.keys()), state="readonly", width=uniform_width)
target_muscle_combobox.grid(column=1, row=2)
target_muscle_combobox.set("BIC")

model_label = ttk.Label(app, text="Model Type:", anchor="w")
model_label.grid(column=0, row=3, sticky="w")
model_combobox = ttk.Combobox(app, values=model_list, state="readonly", width=uniform_width)
model_combobox.grid(column=1, row=3)
model_combobox.set(model_list[0])

excitation_BIC_max_label = ttk.Label(app, text="Excitation BIC Max:", anchor="w")
excitation_BIC_max_label.grid(column=0, row=4, sticky="w")
excitation_BIC_max_var = tk.StringVar()
excitation_BIC_max_var.trace("w", on_value_change)
excitation_BIC_max_entry = ttk.Entry(app, textvariable=excitation_BIC_max_var, width=uniform_width)
excitation_BIC_max_entry.grid(column=1, row=4)
excitation_BIC_max_entry.insert(0, "0.14")

excitation_TRI_max_label = ttk.Label(app, text="Excitation TRI Max:", anchor="w")
excitation_TRI_max_label.grid(column=0, row=5, sticky="w")
excitation_TRI_max_var = tk.StringVar()
excitation_TRI_max_var.trace("w", on_value_change)
excitation_TRI_max_entry = ttk.Entry(app, textvariable=excitation_TRI_max_var, width=uniform_width)
excitation_TRI_max_entry.grid(column=1, row=5)
excitation_TRI_max_entry.insert(0, "0.08")

quiet_time_label = ttk.Label(app, text="Quiet Time:", anchor="w")
quiet_time_label.grid(column=0, row=6, sticky="w")
quiet_time_var = tk.StringVar()
quiet_time_var.trace("w", on_value_change)
quiet_time_entry = ttk.Entry(app, textvariable=quiet_time_var, width=uniform_width)
quiet_time_entry.grid(column=1, row=6)

end_time_label = ttk.Label(app, text="End Time:", anchor="w")
end_time_label.grid(column=0, row=7, sticky="w")
end_time_var = tk.StringVar()
end_time_var.trace("w", on_value_change)
end_time_entry = ttk.Entry(app, textvariable=end_time_var, width=uniform_width)
end_time_entry.grid(column=1, row=7)

time_step_label = ttk.Label(app, text="TIME_STEP:", anchor="w")
time_step_label.grid(column=0, row=8, sticky="w")
time_step_entry = ttk.Entry(app, width=uniform_width)
time_step_entry.grid(column=1, row=8)
time_step_entry.insert(0, "1e-4 * 5")

new_folder_name_label = ttk.Label(app, text="New Folder Name:", anchor="w")
new_folder_name_label.grid(column=0, row=9, sticky="w")
new_folder_name_entry = ttk.Entry(app, width=uniform_width)
new_folder_name_entry.grid(column=1, row=9)
today_date = datetime.datetime.now().strftime("%m%d")
new_folder_name_entry.insert(0, today_date)

# Notes section with predefined notes
notes_label = ttk.Label(app, text="Notes:", anchor="w")
notes_label.grid(column=0, row=10, sticky="w")
notes_text = scrolledtext.ScrolledText(app, width=40, height=10)
notes_text.grid(column=1, row=10)
# ファイルパスを指定
note_path = os.path.join(os.path.dirname(__file__), 'note_for_gui.txt')# ファイルを読み取りモードで開く
with open(note_path, 'r', encoding='utf-8') as file:
    # ファイルの全内容を読み込む
    content = file.read()
predefined_notes = content
notes_text.insert('1.0', predefined_notes)

run_button = ttk.Button(app, text="Run Simulation", command=run_simulation)
run_button.grid(column=0, row=11, columnspan=2)

app.mainloop()